//! Deals with getting all paragraphs for a given law text

use log::info;
use serde::Deserialize;

use crate::misc::{current_date, Error};

use super::parser::OgdSearchResult;

/// # Parameters
/// - `law_id`: Id from RIS of a law
///
/// # Returns
/// Error, or a list of Xml file links of the given law text
pub fn parse(law_id: usize) -> Result<Vec<String>, Error> {
    let mut page = 1;
    let mut skip = true;
    let mut ret = Vec::new();
    loop {
        info!("=== Fetching overview page #{page} ===");
        let json = fetch_page(law_id, page)?;
        let (cont, nodes) = parse_from_str(&json, skip)?;
        for n in nodes {
            ret.push(n.clone());
        }
        if !cont {
            break;
        }
        skip = false;
        page += 1;
    }

    Ok(ret)
}

#[derive(Deserialize)]
#[serde(rename_all = "PascalCase")]
struct Overview {
    ogd_search_result: OgdSearchResult,
}

//TODO: Remove, as we don't want to test internals?
#[cfg(test)]
pub(crate) fn parse_from_str_test(
    content: &str,
    skip_first: bool,
) -> Result<(bool, Vec<String>), Error> {
    parse_from_str(content, skip_first)
}

fn parse_from_str(content: &str, skip_first: bool) -> Result<(bool, Vec<String>), Error> {
    let mut ret = Vec::new();
    let wrapper: Overview = serde_json::from_str(content)?;

    let iter = wrapper.ogd_search_result.get_par().into_iter();
    let boxed_iter: Box<dyn Iterator<Item = String>> = if skip_first {
        Box::new(iter.skip(1))
    } else {
        Box::new(iter)
    };
    for par in boxed_iter {
        ret.push(par);
    }

    if !wrapper.ogd_search_result.has_next_page() {
        return Ok((false, ret));
    }
    Ok((true, ret))
}

/// Fetches the json content of the given overview (`law_id`) from the RIS API.
///
/// # Errors
/// Fails if `ureq` can't create a connection, probably because there's no internet connection? (Or RIS is not online.)
fn fetch_page(overview_id: usize, page: usize) -> Result<String, Error> {
    Ok(
        ureq::post("https://data.bka.gv.at/ris/api/v2.6/Bundesrecht")
            .send_form(&[
                ("Applikation", "BrKons"),
                ("Gesetzesnummer", &format!("{overview_id}")),
                ("DokumenteProSeite", "OneHundred"),
                ("Seitennummer", &format!("{page}")),
                ("Fassung.FassungVom", &current_date()),
            ])?
            .into_string()?,
    )
}
